use actix_web::{
	web, post,
	HttpResponse, HttpRequest, HttpMessage//, Responder
};

use super::types::{
	AppState, MessageForm
};

use crate::lib::users::{
	User, UserCookie//, UserCookieParseError
};

/*
impl From<UserCookieParseError> for HttpResponse {
	fn from(ucpe: UserCookieParseError) -> HttpResponse {
		ucpe.into()
	}
}

//@ line 40: let cookie = UserCookie::parse(cookie.unwrap().value())?;
*/
/*
 * For some reason the compiler lets me implicitly convert from UserCookieParseError
 * even though (as far as I know) have no real way of (implicitly) converting between the two
 *
 * Down the road this causes a core dump following a stack overflow.
 */

#[post("/messages/new")]
pub async fn new(
	req: HttpRequest,
	form: web::Form<MessageForm>,
	data: web::Data<AppState>
) -> Result<HttpResponse, HttpResponse> {
	let cookie = req.cookie("auth");
	if cookie.is_none() {
		return Err(HttpResponse::Unauthorized().body("Unauthorized: You are not logged in"));
	}
	let cookie = match UserCookie::parse(cookie.unwrap().value()) {
		Ok(a) => a,
		Err(e) => {
			println!("{}", e);
			return Err(HttpResponse::BadRequest().body("Bad Request: Could not parse cookie data"));
		}
	};
	let mut mf = data.msg_factory.lock().unwrap();
	let b = data.boards.lock().unwrap();
	let board = b.iter().find(|x| (x.id == form.board_id));
	let um = data.user_manager.lock().unwrap();
	match board {
		Some(a) => {
			let msg = mf.create_message({
				let user = um.get_user_by_id(form.user_id).ok_or_else(|| {
					HttpResponse::NotFound().body("Not Found: User does not exist")
				})?;
				if user.secret == cookie.secret {
					User::from(user)
				} else {
					return Err(HttpResponse::Unauthorized().body("Unauthorized: Bad login attempt, invalid auth token"));
				}
			}, form.content.clone());
			println!("new message");
			a.add_message(msg.clone());
			Ok(HttpResponse::Created().body(serde_json::to_string(&msg).unwrap()))
		},
		None => Err(HttpResponse::BadRequest().body("Bad Request: Board does not exist"))
	}
}
