use std::sync::Mutex;

use serde::{
	Deserialize, Serialize
};

use crate::lib::{
	boards::{BoardFactory, Board},
	messages::{MessageFactory},
	users::UserManager
};

#[derive(Debug, Serialize, Deserialize)]
pub struct AppState {
	pub boards: Mutex<Vec<Board>>,
	pub board_factory: Mutex<BoardFactory>,
	pub msg_factory: Mutex<MessageFactory>,
	pub user_manager: Mutex<UserManager>,
}

#[derive(Serialize, Deserialize)]
pub struct BoardForm {
	pub name: String,
	pub description: String,
	pub author: Option<u32>
}

#[derive(Deserialize)]
pub struct PageForm {
	pub page: u16
}

#[derive(Serialize, Deserialize)]
pub struct MessageForm {
	pub content: String,
	pub board_id: u32,
	pub user_id: u32
}

#[derive(Debug, Serialize, Deserialize)]
pub struct UserForm {
	pub nick: Option<String>,
	pub id: Option<u32>
}

#[derive(Debug, Serialize, Deserialize)]
pub struct CryptoUserForm {
	pub username: String,
	pub password: String
}

#[derive(Debug, Serialize, Deserialize)]
pub struct NickForm {
	pub nickname: String
}
