//use std::fmt::Display;
use crate::lib::users::{User, Account};

use super::types::{AppState, UserForm, CryptoUserForm};//, NickForm};

use actix_web::{
	web, get, post,
	http::{
		header::ContentType,
		Cookie,
	},
	HttpResponse//, HttpRequest, HttpMessage
};

#[post("/users/new")]
pub async fn new(
	data: web::Data<AppState>,
	form: web::Form<CryptoUserForm>
) -> HttpResponse {
	println!("new user");
	let mut um = data.user_manager.lock().unwrap();
	let user = {
		if let Some(_a) = um.get_user_by_uname(form.username.clone()) {
			return HttpResponse::Conflict().body("Conflict: user with that username already exists");
		} else {
			let new_id = um.assign_id();
			User::new(new_id, &form.username).encrypt(form.password.clone())
		}
	};
	//HttpResponse::Created().body("Created: Created user")
	um.add_user(user.clone());
	HttpResponse::Created().body(serde_json::to_string(&User::from(user)).unwrap())
}

#[get("/users/list")]
pub async fn list(
	data: web::Data<AppState>
) -> HttpResponse {
	let v: Vec<User> = data.user_manager.lock().unwrap().users.clone().into_iter().map(User::from).collect();
	HttpResponse::Ok().set(ContentType::json()).body(
		serde_json::to_string(&v).unwrap()
	)
}

#[get("/users/get")]
pub async fn get(
	data: web::Data<AppState>,
	form: web::Form<UserForm>
) -> HttpResponse {
	let um = data.user_manager.lock().unwrap();
	if let Some(id) = form.id {
		match um.get_user_by_id(id) {
			Some(a) => return HttpResponse::Found()
				.set(ContentType::json())
				.body(serde_json::to_string_pretty(&User::from(a)).unwrap()),
			None => return HttpResponse::NotFound().body("Not Found: That user could not be found")
		}
	} else if let Some(nick) = &form.nick {
		match um.get_user_by_nick(nick.to_string()) {
			Some(a) => return HttpResponse::Found()
				.set(ContentType::json())
				.body(serde_json::to_string_pretty(&User::from(a)).unwrap()),
			None => return HttpResponse::NotFound().body("Not Found: That user could not be found")
		}
	} else {
		HttpResponse::BadRequest().body("Bad Request: Bad form data")
	}
}

#[post("/users/auth")]
pub async fn auth(
	data: web::Data<AppState>,
	form: web::Form<CryptoUserForm>
) -> HttpResponse {
	let user = data.user_manager.lock().unwrap().get_user_by_uname(form.username.clone());
	if let Some(u) = user {
		match u.verify(form.password.clone()) {
			Ok(a) => if a {
				let mut response = HttpResponse::Ok()
					.body(serde_json::to_string(&User::from(u.clone())).unwrap());
				response.add_cookie(
					&Cookie::build("auth", format!("{}&{}", u.user_id, u.secret))
						.domain("boards.solow.xyz")
						//.domain("localhost")
						.path("/")
						.same_site(actix_web::cookie::SameSite::Strict)
						//.secure(true)
						.finish()
				).unwrap();
				response
			} else {
				HttpResponse::Forbidden().body("Forbidden: Could not authenticate user")
			}
			Err(e) => HttpResponse::BadRequest().body(format!("Bad Request: {}", e))
		}
	} else {
		HttpResponse::BadRequest().body("Bad Request: User does not exist")
	}
}

/*
#[post("/users/set_nick")]
pub async fn set_nick(
	req: HttpRequest,
	data: web::Data<AppState>,
	form: web::Form<NickForm>
) -> HttpResponse {
	if let Some(cookie) = req.cookie("auth") {
		HttpResponse::Ok().body("")
	} else {
		HttpResponse::Unauthorized().body("Unauthorized: You are not logged in")
	}
}
*/
