use actix_web::{
	get, post, web,
	HttpResponse, Responder,
	cookie::Cookie
};

use crate::api::types::*;
use crate::lib::users::{User, Account};

#[post("/account/auth")]
pub async fn auth_html(
	data: web::Data<AppState>,
	form: web::Form<CryptoUserForm>
) -> HttpResponse {
	let user = data.user_manager.lock().unwrap().get_user_by_uname(form.username.clone());
	if let Some(u) = user {
		match u.verify(form.password.clone()) {
			Ok(a) => if a {
				let mut response = HttpResponse::Ok()
					.body(format!(r#"
<html><head>
<link rel="stylesheet" type="text/css" href="https://orbitalfox.us/Music/index.css">
<link rel="stylesheet" type="text/css" href="/static/index.css">
</head><body>
<div id="title" class="bar">
<h1>Hello, {0}</h1>
</div><div id="index">
<p>
Logged in as {0}, return to <a href="/boards/">index</a>
</p>
</body></html></div></body></html>
"#, u.username));
				response.add_cookie(
					&Cookie::build("auth", format!("{}&{}", u.user_id, u.secret))
						.domain("boards.solow.xyz")
						//.domain("localhost")
						.path("/")
						.same_site(actix_web::cookie::SameSite::Strict)
						//.secure(true)
						.finish()
				).unwrap();
				response
			} else {
				HttpResponse::Forbidden().body("Forbidden: Could not authenticate user")
			}
			Err(e) => HttpResponse::BadRequest().body(format!("Bad Request: {}", e))
		}
	} else {
		HttpResponse::BadRequest().body("Bad Request: User does not exist")
	}
}

#[get("/account/login/")]
pub async fn login() -> impl Responder {
	HttpResponse::Ok().body(r#"
<html><head>
<link rel="stylesheet" type="text/css" href="https://orbitalfox.us/Music/index.css">
<link rel="stylesheet" type="text/css" href="/static/index.css">
</head><body>
<div id="title" class="bar">
<h1>Login</h1>
</div><div id="index">
<div class="auth">
<form action="/account/auth" method="post">
<div>
<label for="username">Username: </label>
<input type="text" id="username" name="username" required>
</div>
<div>
<label for="password">Password: </label>
<input type="password" id="password" name="password" required>
</div>
<input type="submit" value="Login">
</form>
</div>
</body></html></div></body></html>
"#)
}

#[post("/account/new")]
pub async fn sign_up_result(
	data: web::Data<AppState>,
	form: web::Form<CryptoUserForm>
) -> HttpResponse {
	println!("new user");
	let mut um = data.user_manager.lock().unwrap();
	let user = {
		if let Some(_a) = um.get_user_by_uname(form.username.clone()) {
			return HttpResponse::Conflict().body("Conflict: user with that username already exists");
		} else {
			let new_id = um.assign_id();
			User::new(new_id, &form.username).encrypt(form.password.clone())
		}
	};
	um.add_user(user.clone());
	HttpResponse::Created().body(format!(r#"
<html><head>
<link rel="stylesheet" type="text/css" href="https://orbitalfox.us/Music/index.css">
<link rel="stylesheet" type="text/css" href="/static/index.css">
</head><body>
<div id="title" class="bar">
<h1>Hello, {0}</h1>
</div><div id="index">
<p>
Hooray! You can log into you new account <a href="/account/login/">here</a>
</p>
</body></html></div></body></html>
"#, user.username))
}

#[get("/account/new/")]
pub async fn sign_up() -> impl Responder {
	HttpResponse::Ok().body(r#"
<html><head>
<link rel="stylesheet" type="text/css" href="https://orbitalfox.us/Music/index.css">
<link rel="stylesheet" type="text/css" href="/static/index.css">
</head><body>
<div id="title" class="bar">
<h1>Sign Up</h1>
</div><div id="index">
<div class="auth">
<form action="/account/new" method="post">
<div>
<label for="username">Username: </label>
<input type="text" id="username" name="username" required>
</div>
<div>
<label for="password">Password: </label>
<input type="password" id="password" name="password" required>
</div>
<input type="submit" value="Sign Up">
</form>
</div>
</body></html></div></body></html>
"#)
}
