use serde::{Serialize, Deserialize};
use std::{
	cell::RefCell,
	time::SystemTime,
	path::Path
};

use super::{
	Archiveable,
	users::User
};

#[derive(Serialize, Deserialize, Debug, Clone)]
pub struct Message {
	pub text: String,
	pub id: u32,
	pub timestamp: u64,
	pub author: RefCell<User>
}

#[derive(Debug, Serialize, Deserialize)]
pub struct MessageFactory {
	pub next_id: u32,
	pub last_write: Option<u64>,
	pub data_dir: String
}

impl MessageFactory {
	pub fn new<P>(path: P) -> MessageFactory where P: AsRef<Path> {
		MessageFactory {
			next_id: 0,
			last_write: None,
			data_dir: path.as_ref().to_str().unwrap().into()
		}
	}

	pub fn create_message<S>(&mut self, user: User, content: S) -> Message where S: AsRef<str> {
		self.next_id += 1;
		let time = SystemTime::now().duration_since(SystemTime::UNIX_EPOCH).unwrap().as_secs();
		Message {
			text: String::from(content.as_ref()),
			id: self.next_id-1,
			timestamp: time,
			author: RefCell::new(user)
		}
	}
}

impl Archiveable<Self> for MessageFactory {
	fn get_root(&self) -> &Path {
		Path::new(&self.data_dir)
	}
}
